/* Copyright 2015-2017 MultiMC Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include "BaseVersion.h"

#include <QJsonObject>
#include <QStringList>
#include <QVector>
#include <memory>

#include "minecraft/VersionFile.h"

#include "BaseEntity.h"

#include "multimc_logic_export.h"

namespace Meta
{
using VersionPtr = std::shared_ptr<class Version>;

class MULTIMC_LOGIC_EXPORT Version : public QObject, public BaseVersion, public BaseEntity
{
	Q_OBJECT

public: /* con/des */
	explicit Version(const QString &uid, const QString &version);

	QString descriptor() override;
	QString name() override;
	QString typeString() const override;

	QString uid() const
	{
		return m_uid;
	}
	QString parentUid() const
	{
		return m_parentUid;
	}
	QString version() const
	{
		return m_version;
	}
	QString type() const
	{
		return m_type;
	}
	QDateTime time() const;
	qint64 rawTime() const
	{
		return m_time;
	}
	const QHash<QString, QString> &requires() const
	{
		return m_requires;
	}
	VersionFilePtr data() const
	{
		return m_data;
	}
	bool isRecommended() const
	{
		return m_recommended;
	}

	void merge(const std::shared_ptr<BaseEntity> &other) override;
	void parse(const QJsonObject &obj) override;

	QString localFilename() const override;

public: // for usage by format parsers only
	void setParentUid(const QString &parentUid);
	void setType(const QString &type);
	void setTime(const qint64 time);
	void setRequires(const QHash<QString, QString> &requires);
	void setRecommended(bool recommended);
	void setProvidesRecommendations();
	void setData(const VersionFilePtr &data);

signals:
	void typeChanged();
	void timeChanged();
	void requiresChanged();

private:
	bool m_providesRecommendations = false;
	bool m_recommended = false;
	QString m_name;
	QString m_uid;
	QString m_parentUid;
	QString m_version;
	QString m_type;
	qint64 m_time;
	QHash<QString, QString> m_requires;
	VersionFilePtr m_data;
};
}

Q_DECLARE_METATYPE(Meta::VersionPtr)
